@@ -79,6 +79,81 @@ static gboolean opt_nodisplay = FALSE;
 
 #define DBUS_TIMEOUT G_MAXINT
 
+static gboolean
+attempt_to_gain_privilege (const char *error_detail)
+{
+        const char *pk_action;
+        const char *pk_result;
+        char **tokens;
+        gboolean ret;
+        DBusGConnection *session_bus;
+        DBusGProxy *polkit_gnome_proxy;
+        GError *g_error = NULL;
+        gboolean gained_privilege;
+
+        ret = FALSE;
+        tokens = NULL;
+        polkit_gnome_proxy = NULL;
+
+        /* return FALSE if not in GUI mode */
+        if (opt_noui)
+                goto out;
+
+        tokens = g_strsplit (error_detail, " ", 0);
+        if (tokens == NULL)
+                goto out;
+        if (g_strv_length (tokens) < 2)
+                goto out;
+        pk_action = tokens[0];
+        pk_result = tokens[1];
+
+        g_debug ("pk_action='%s' pk_result='%s'", pk_action, pk_result);
+
+        session_bus = dbus_g_bus_get (DBUS_BUS_SESSION, &g_error);
+        if (session_bus == NULL) {
+                g_warning ("Caught exception '%s'", g_error->message);
+                g_error_free (g_error);
+                goto out;
+        }
+	polkit_gnome_proxy = dbus_g_proxy_new_for_name (session_bus,
+                                                        "org.gnome.PolicyKit",           /* bus name */
+                                                        "/org/gnome/PolicyKit/Manager",  /* object */
+                                                        "org.gnome.PolicyKit.Manager");  /* interface */
+        if (polkit_gnome_proxy == NULL) {
+                goto out;
+        }
+
+        /* now use PolicyKit-gnome to bring up an auth dialog (we
+         * don't have any windows so set the XID to "null") 
+         */
+        if (!dbus_g_proxy_call_with_timeout (polkit_gnome_proxy,
+                                             "ShowDialog",
+                                             INT_MAX,
+                                             &g_error,
+                                             /* parameters: */
+                                             G_TYPE_STRING, pk_action,      /* action_id */
+                                             G_TYPE_UINT, 0,                /* X11 window ID */
+                                             G_TYPE_INVALID,
+                                             /* return values: */
+                                             G_TYPE_BOOLEAN, &gained_privilege,
+                                             G_TYPE_INVALID)) {
+                g_warning ("Caught exception '%s'", g_error->message);
+                g_error_free (g_error);
+                goto out;
+        }
+
+        ret = gained_privilege;
+        g_debug ("gained privilege = %d", gained_privilege);
+
+out:
+        if (polkit_gnome_proxy != NULL)
+                g_object_unref (polkit_gnome_proxy);
+        if (tokens != NULL)
+                g_strfreev (tokens);
+
+        return ret;
+}
+
 static void 
 notify_parent (gboolean success)
 {
@@ -398,6 +473,17 @@ show_error_dialog_mount (LibHalVolume *volume, LibHalDrive *drive,
 			/* show gory details */
 			show_details = TRUE;
 #endif
+		} else if (strcmp (error_name, "org.freedesktop.Hal.Device.InterfaceLocked") == 0) {
+			/* this happens when someone (such as gparted or anaconda) is locking
+			 *
+			 * Don't show any dialog for this; because
+			 * it's probably locked due to partitioning
+			 * and the volume contains signatures are not
+			 * yet cleaned out...
+			 */
+			gtk_widget_unref (w);
+			w = NULL;
+			 
 		} else if (strcmp (error_name, "org.freedesktop.DBus.Error.UnknownMethod") == 0) {
 			/* happens when HAL don't support the Mount() method - which we only 
 			 * do for volumes with mountable file systems
@@ -727,7 +813,14 @@ volume_mount_with_options (const char *udi, LibHalVolume *volume, LibHalDrive *d
 			dbus_error_free (&error);
 
 			return ret;
-		}
+		} else if (strcmp (error.name, "org.freedesktop.Hal.Device.PermissionDeniedByPolicy") == 0) {
+                        /* attempt to gain privilege */
+                        if (attempt_to_gain_privilege (error.message)) {
+                                ret = volume_mount_with_options (udi, volume, drive, NULL, fstype, options);
+                        }
+                        dbus_error_free (&error);
+                        return ret;
+                }
 
 		notify_parent (FALSE);
 
@@ -1726,6 +1819,16 @@ volume_unmount (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 			unmount_cache_timeout_cancel (FALSE);
 		}
 		g_warning ("Unmount failed for %s: %s : %s\n", udi, error.name, error.message);
+
+		if (strcmp (error.name, "org.freedesktop.Hal.Device.PermissionDeniedByPolicy") == 0) {
+                        /* attempt to gain privilege */
+                        if (attempt_to_gain_privilege (error.message)) {
+                                ret = volume_unmount (udi, volume, drive);
+                        }
+                        dbus_error_free (&error);
+                        return ret;
+                }
+
 		show_error_dialog_unmount (udi, volume, drive, error.name, error.message);
 		dbus_error_free (&error);
 		goto out;
@@ -1930,6 +2033,16 @@ volume_eject (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 			unmount_cache_timeout_cancel (FALSE);
 		}
 		g_warning ("Eject failed for %s: %s : %s\n", udi, error.name, error.message);
+
+		if (strcmp (error.name, "org.freedesktop.Hal.Device.PermissionDeniedByPolicy") == 0) {
+                        /* attempt to gain privilege */
+                        if (attempt_to_gain_privilege (error.message)) {
+                                ret = volume_eject (udi, volume, drive);
+                        }
+                        dbus_error_free (&error);
+                        return ret;
+                }
+
 		show_error_dialog_eject (udi, volume, drive, error.name, error.message);
 		dbus_error_free (&error);
 		goto out;
@@ -2098,7 +2211,7 @@ get_password (const char *udi, LibHalVolume *volume, LibHalDrive *drive, gboolea
 	notify_parent (TRUE);
 
 	prompt = g_strdup_printf (
-		_("The storage device%s contains encrypted data. Enter a password to unlock."), drive_name);
+		_("The storage device %s contains encrypted data. Enter a password to unlock."), drive_name);
 	
 	dialog = gnome_password_dialog_new (_("Unlock Encrypted Data"), prompt, NULL, NULL, FALSE);
 	g_free (prompt);
